package net.silnote.bss.references;

/**
 * Created with IntelliJ IDEA.
 * User: Александр
 * Date: 21.02.13
 * Time: 17:44
 * To change this template use File | Settings | File Templates.
 */

import com.intellij.codeInsight.lookup.LookupElement;
import com.intellij.codeInsight.lookup.LookupElementBuilder;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.TextRange;
import com.intellij.psi.*;
import com.intellij.psi.util.PsiTreeUtil;
import net.silnote.bss.BSSUtil;
import net.silnote.bss.Psi.impl.BSSIdentifireImpl;
import net.silnote.bss.Psi.impl.BSSVarDefinitionImpl;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

public class BSSReference extends PsiReferenceBase<PsiElement> implements PsiPolyVariantReference {
    private String key;
    private BSSIdentifireImpl element;

    public BSSReference(@NotNull PsiElement element, TextRange textRange) {
        super(element, textRange);
        this.element = (BSSIdentifireImpl)element;
        key = this.element.getName();
    }

    @NotNull
    @Override
    public ResolveResult[] multiResolve(boolean incompleteCode) {
        List<ResolveResult> results = new ArrayList<ResolveResult>();
        System.out.println("BSSIdent multiResolve");
        if(element.isMainIdentifire())
        {
            return ResolveResult.EMPTY_ARRAY;
        }
        else {
            PsiElement parent = BSSUtil.closestFunction(element);
            if(parent != null) {
                PsiElement[] elements = PsiTreeUtil.getChildrenOfType(parent, BSSVarDefinitionImpl.class);
                if (elements != null) {
                    for (PsiElement el : elements) {
                        PsiElement[] identifire = PsiTreeUtil.getChildrenOfType(el, BSSIdentifireImpl.class);
                        System.out.println("multiResolve " + el);
                        if(identifire != null) {
                            if(((BSSIdentifireImpl)identifire[0]).getName().equals(element.getName())) {
                                System.out.println("multiResolve " + ((BSSIdentifireImpl)identifire[0]).getName());
                                results.add(new PsiElementResolveResult(identifire[0]));
                            }
                        }
                    }
                }
            }
        }
        return results.toArray(new ResolveResult[results.size()]);
    }

    @Nullable
    @Override
    public PsiElement resolve() {
        System.out.println("resolve");
        ResolveResult[] resolveResults = multiResolve(false);
        return resolveResults.length == 1 ? resolveResults[0].getElement() : null;
    }

    @NotNull
    @Override
    public Object[] getVariants() {
        System.out.println("getVariants");
        Project project = myElement.getProject();
        List<LookupElement> variants = new ArrayList<LookupElement>();
        List<PsiElement> list = BSSUtil.findElementsByName(project, element.getName());
        for(PsiElement l : list) {
            variants.add(LookupElementBuilder.create((PsiNamedElement)l));
        }
        return variants.toArray();
    }

    @Override
    public boolean isSoft() {
        return true;
    }

    @Override
    public boolean isReferenceTo(PsiElement element) {
        System.out.println("isReferenceTo");
        return resolve() == element;
    }
}